//  @flow
import 'babel-polyfill';
import React from 'react';
import { BrowserRouter } from 'react-router-dom';

import type { IConfig } from '../flow/IConfig';

//  This ensures any serverProps are passed to React
//  on the first initial load
const getClientProps = () => {
  try {
    const scriptsEl = document.getElementById('app-props');
    let props = scriptsEl ? scriptsEl.textContent : {};

    //  eslint-disable-next-line
    props = props.replace("<![CDATA[", "").replace("]]>", "");
    return JSON.parse(props);
  } catch (err) {
    throw new Error('Error parsing client props');
  }
};

//  This code block creates a hot-module-reloading
//  subscription for the extracted CSS bundle
//  Since we can't have a FOUC during initial page render, when
//  the extracted CSS bundle is updated, we receive an event and new styles
//  are injected into the <head />
if (module && module.hot) {
  const head = document.head || document.getElementsByTagName('head')[0];
  const hotInlineStyles = document.createElement('style');
  hotInlineStyles.type = 'text/css';
  head.appendChild(hotInlineStyles);

  require('webpack-hot-middleware/client').subscribe((updateEvent) => {
    if (updateEvent.type === 'css-update') {
      hotInlineStyles.textContent = '';
      hotInlineStyles.appendChild(document.createTextNode(updateEvent.value));
    }
  });
}

export const createClient = (config: IConfig) => {
  const serverProps = getClientProps();

  const { root: Root } = config;
  return (
    <BrowserRouter>
      <Root {...serverProps} />
    </BrowserRouter>
  );
};

export default createClient;
